@@ -44,8 +44,11 @@
 #include <synfig/pair.h>
 #include <synfig/waypoint.h>
 #include <synfig/valuenode_registry.h>
+#include <synfig/surface.h>
+#include <synfig/rendering/software/surfacesw.h>
 
 #include <synfig/layers/layer_pastecanvas.h>
+#include <synfig/layers/layer_bitmap.h>
 
 #include <synfig/valuenodes/valuenode_animatedfile.h>
 #include <synfig/valuenodes/valuenode_animatedfile.h>
@@ -78,7 +81,6 @@
 
 /* === U S I N G =========================================================== */
 
-using namespace std;
 using namespace etl;
 using namespace synfig;
 using namespace synfigapp;
@@ -95,7 +97,8 @@ CanvasInterface::CanvasInterface(etl::loose_handle<Instance> instance,etl::handl
 	instance_(instance),
 	canvas_(canvas),
 	cur_time_(canvas->rend_desc().get_frame_start()),
-	mode_(MODE_NORMAL|MODE_ANIMATE_PAST|MODE_ANIMATE_FUTURE)
+	mode_(MODE_NORMAL|MODE_ANIMATE_PAST|MODE_ANIMATE_FUTURE),
+	state_("")
 {
 	set_selection_manager(get_instance()->get_selection_manager());
 	set_ui_interface(get_instance()->get_ui_interface());
@@ -185,6 +188,17 @@ CanvasInterface::get_mode()const
 	return mode_;
 }
 
+void
+CanvasInterface::set_state(synfig::String x) {
+	state_=x;
+}
+
+synfig::String
+CanvasInterface::get_state()const
+{
+	return state_;
+}
+
 synfig::Layer::Handle
 CanvasInterface::layer_create(
 	const synfig::String &id,
@@ -294,10 +308,10 @@ CanvasInterface::layer_set_defaults(const synfig::Layer::Handle &layer)
 			if(iter->second.get_type()==type_list)
 			{
 				// check whether it's a list of blinepoints or widthpoints only
-				vector<ValueBase> list(iter->second.get_list());
+				std::vector<ValueBase> list(iter->second.get_list());
 				if (list.size())
 				{
-					vector<ValueBase>::iterator iter2 = list.begin();
+					std::vector<ValueBase>::iterator iter2 = list.begin();
 					Type &type(iter2->get_type());
 					for (iter2++; iter2 != list.end(); iter2++)
 						if (iter2->get_type() != type)
@@ -711,7 +725,7 @@ CanvasInterface::import(
 	String short_filename = CanvasFileNaming::make_short_filename(get_canvas()->get_file_name(), filename);
 	String full_filename = CanvasFileNaming::make_full_filename(get_canvas()->get_file_name(), short_filename);
 
-	if (ext=="pgo")
+	if (ext=="pgo" || ext=="tsv" || ext=="xml")
 	{
 		synfigapp::Action::PassiveGrouper group(get_instance().get(),_("Import Lipsync"));
 
@@ -942,7 +956,8 @@ CanvasInterface::import_sequence(
 	const std::set<synfig::String> &filenames,
 	synfig::String &errors,
 	synfig::String &/*warnings*/,
-	bool resize_image )
+	bool resize_image,
+	bool remove_dups)
 {
 	Action::PassiveGrouper group(get_instance().get(),_("Import sequence"));
 
@@ -954,16 +969,25 @@ CanvasInterface::import_sequence(
 	try {
 		// add imported layers into switch
 		Action::Handle action(Action::create("LayerEncapsulateSwitch"));
+		Action::Handle d_action(synfigapp::Action::LayerRemove::create());
 		if(!action)
 			{ get_ui_interface()->error(_("Cannot create action")); throw int(); }
 		action->set_param("canvas",get_canvas());
 		action->set_param("canvas_interface",etl::loose_handle<CanvasInterface>(this));
+		d_action->set_param("canvas",get_canvas());
+		d_action->set_param("canvas_interface",etl::loose_handle<CanvasInterface>(this));
 
 		// create layers and assign them with LayerEncapsulateSwitch action
 		Layer::Handle layer;
+		Layer_Bitmap::Handle curr_layer;
+		rendering::Surface::Handle  cur_surface,prev_surface= rendering::Surface::Handle();
+		bool first_time=true;
 		int layers_count = 0;
-		for(std::set<String>::const_iterator i = filenames.begin(); i != filenames.end(); ++i) {
-			const String &filename = *i;
+		std::set<String>::const_iterator c1= filenames.begin();
+		std::set<String>::const_iterator c2= filenames.begin();
+
+		while(c2!=filenames.end()){
+			const String &filename = *c2;
 			synfig::info("Attempting to import '%s' into sequence", filename.c_str());
 			
 			String ext(filename_extension(filename));
@@ -991,6 +1015,29 @@ CanvasInterface::import_sequence(
 					throw int();
 				if (!layer->set_param("filename", ValueBase(short_filename)))
 					throw int();
+
+				if(remove_dups){
+					curr_layer= Layer_Bitmap::Handle::cast_dynamic(layer);
+					if(!curr_layer){
+						throw int();
+					}
+					rendering::SurfaceResource::LockRead<rendering::SurfaceSW> cur_lock(curr_layer->rendering_surface);
+					if(!cur_lock) {
+						throw int();
+					}
+					cur_surface=cur_lock.get_handle();
+
+					if(c2!=std::next(filenames.end(), -1) && !first_time && cur_surface->compare_with(prev_surface)){
+						d_action->set_param("layer",layer);
+						advance(c2,1);
+						++layers_count;
+						continue;
+					}
+					if(first_time) {
+						first_time = false;
+					}
+				}
+
 				w = layer->get_param("_width").get(int());
 				h = layer->get_param("_height").get(int());
 				layer->monitor(filename);
@@ -1013,7 +1060,6 @@ CanvasInterface::import_sequence(
 						if((size[0]<0)) x[0]=-x[0];
 						if((size[1]<0)) x[1]=-x[1];
 					}
-
 					if(!layer->set_param("tl",ValueBase(-x/2)))
 						throw int();
 					if(!layer->set_param("br",ValueBase(x/2)))
@@ -1024,36 +1070,42 @@ CanvasInterface::import_sequence(
 					if(!layer->set_param("br",ValueBase(get_canvas()->rend_desc().get_br())))
 						throw int();
 				}
-				
+
 				String desc = etl::basename(filename);
 				layer->set_description(desc);
 				signal_layer_new_description()(layer, desc);
-				
+
 				action->set_param("layer", layer);
 				if (!layers_count)
 					action->set_param("description", desc);
-				
+
 				Waypoint &wp = *layer_name_animated->new_waypoint(Time(layers_count/fps), ValueBase(desc));
 				wp.set_before(INTERPOLATION_CONSTANT);
 				wp.set_after(INTERPOLATION_CONSTANT);
-				
+
 				++layers_count;
+				c1=c2;
+				prev_surface=cur_surface;
+				advance(c2,1);
 			} catch(...) {
 				errors += etl::strprintf(_("Unable to import file: %s"), filename.c_str());
 				group.cancel();
 				return false;
 			}
 		}
-		
 		if (!layers_count)
 			{ get_ui_interface()->error(_("Nothing imported")); throw int(); }
 		if(!action->is_ready())
 			{ get_ui_interface()->error(_("Action Not Ready")); throw int(); }
 		if(!get_instance()->perform_action(action))
 			{ get_ui_interface()->error(_("Action Failed.")); throw int(); }
+		if(remove_dups){
+			if(!get_instance()->perform_action(d_action))
+				{ get_ui_interface()->error(_("Delete Action Failed.")); throw int(); }
+		}
 
 		if (layer) {
-			// get parent layer, because image is incapsulated into action switch
+			// get parent layer, because image is encapsulated into action switch
 			Layer::Handle layer_switch = layer->get_parent_paste_canvas_layer();
 
 			// connect animated layer_name param
@@ -1294,7 +1346,7 @@ CanvasInterface::set_meta_data(const synfig::String& key,const synfig::String& d
 	if (get_canvas()->get_meta_data(key) == data)
 		return;
 
-    if (key=="guide_x" || key=="guide_y")
+	if (key=="guide_x" || key=="guide_y")
 	{
 		// Create an undoable action
 
@@ -1480,3 +1532,4 @@ CanvasInterface::seek_time(synfig::Time time)
 		newtime=get_canvas()->rend_desc().get_time_end();
 	set_time(newtime);
 }
+
